@@ -35,12 +35,7 @@
 #include "nm-utils/nm-test-utils.h"
 
 #define SRCDIR TEST_SRCDIR"/conf"
-
-#if ((NETWORKMANAGER_COMPILATION) & NM_NETWORKMANAGER_COMPILATION_WITH_LIBNM_UTIL)
-#define TMPDIR TEST_BUILDDIR"/conf-tmp-old"
-#else
-#define TMPDIR TEST_BUILDDIR"/conf-tmp-new"
-#endif
+#define TMPDIR TEST_BUILDDIR"/conf-tmp"
 
 /*****************************************************************************/
 
@@ -226,6 +221,7 @@ test_password_import (void)
 	_check_item (s_vpn, NM_OPENVPN_KEY_TA, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_TA_DIR, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_CIPHER, "AES-256-CBC");
+	_check_item (s_vpn, NM_OPENVPN_KEY_DATA_CIPHERS, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_AUTH, NULL);
@@ -319,6 +315,7 @@ test_tls_import (void)
 	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY_DIRECTION, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_CIPHER, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_DATA_CIPHERS, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_AUTH, NULL);
@@ -366,6 +363,7 @@ test_tls_import_2 (void)
 	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY_DIRECTION, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_CIPHER, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_DATA_CIPHERS, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_AUTH, NULL);
@@ -410,6 +408,7 @@ test_tls_import_3 (void)
 	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY_DIRECTION, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_CIPHER, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_DATA_CIPHERS, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_AUTH, NULL);
@@ -431,6 +430,56 @@ test_tls_import_3 (void)
 
 }
 
+static void
+test_tls_import_4 (void)
+{
+	_CREATE_PLUGIN (plugin);
+	gs_unref_object NMConnection *connection = NULL;
+	NMSettingConnection *s_con;
+	NMSettingVpn *s_vpn;
+
+	connection = get_basic_connection (plugin, SRCDIR, "tls4.ovpn");
+
+	s_con = _get_setting_connection (connection);
+	g_assert_cmpstr (nm_setting_connection_get_id (s_con), ==, "tls4");
+	g_assert (!nm_setting_connection_get_uuid (s_con));
+
+	s_vpn = _get_setting_vpn (connection);
+
+	_check_item (s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE, NM_OPENVPN_CONTYPE_TLS);
+	_check_item (s_vpn, NM_OPENVPN_KEY_DEV, "tun");
+	_check_item (s_vpn, NM_OPENVPN_KEY_PROTO_TCP, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO, "adaptive");
+	_check_item (s_vpn, NM_OPENVPN_KEY_FLOAT, "yes");
+	_check_item (s_vpn, NM_OPENVPN_KEY_RENEG_SECONDS, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE, "173.8.149.245:1194");
+	_check_item (s_vpn, NM_OPENVPN_KEY_PORT, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY_DIRECTION, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_CIPHER, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_DATA_CIPHERS, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_AUTH, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_TLS_REMOTE, "/CN=myvpn.company.com");
+	_check_item (s_vpn, NM_OPENVPN_KEY_VERIFY_X509_NAME,
+	             "subject:C=US, L=Cambridge, CN=GNOME, emailAddress=networkmanager-list@gnome.org");
+	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE_CERT_TLS, "server");
+
+	_check_item (s_vpn, NM_OPENVPN_KEY_CA,           SRCDIR"/keys/mg8.ca");
+	_check_item (s_vpn, NM_OPENVPN_KEY_CERT,         SRCDIR"/keys/clee.crt");
+	_check_item (s_vpn, NM_OPENVPN_KEY_KEY,          SRCDIR"/keys/clee.key");
+	_check_item (s_vpn, NM_OPENVPN_KEY_TLS_CRYPT_V2, SRCDIR"/keys/46.key");
+
+	_check_secret (s_vpn, NM_OPENVPN_KEY_PASSWORD, NULL);
+	_check_secret (s_vpn, NM_OPENVPN_KEY_CERTPASS, NULL);
+
+	_check_item (s_vpn, NM_OPENVPN_KEY_TLS_VERSION_MIN, "1.0");
+	_check_item (s_vpn, NM_OPENVPN_KEY_TLS_VERSION_MIN_OR_HIGHEST, "yes");
+	_check_item (s_vpn, NM_OPENVPN_KEY_TLS_VERSION_MAX, "1.2");
+
+}
+
 static void
 test_file_contents (const char *id,
                     const char *dir,
@@ -481,6 +530,7 @@ test_tls_inline_import (void)
 	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY_DIRECTION, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_CIPHER, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_DATA_CIPHERS, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_AUTH, NULL);
@@ -533,6 +583,7 @@ test_pkcs12_import (void)
 	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY_DIRECTION, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_CIPHER, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_DATA_CIPHERS, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_AUTH, NULL);
@@ -545,6 +596,48 @@ test_pkcs12_import (void)
 	_check_secret (s_vpn, NM_OPENVPN_KEY_CERTPASS, NULL);
 }
 
+static void
+test_pkcs12_with_ca_import (void)
+{
+	_CREATE_PLUGIN (plugin);
+	gs_unref_object NMConnection *connection = NULL;
+	NMSettingConnection *s_con;
+	NMSettingVpn *s_vpn;
+	const char *expected_id = "pkcs12-with-ca";
+
+	connection = get_basic_connection (plugin, SRCDIR, "pkcs12-with-ca.ovpn");
+
+	s_con = _get_setting_connection (connection);
+	g_assert_cmpstr (nm_setting_connection_get_id (s_con), ==, expected_id);
+	g_assert (!nm_setting_connection_get_uuid (s_con));
+
+	s_vpn = _get_setting_vpn (connection);
+
+	_check_item (s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE, NM_OPENVPN_CONTYPE_TLS);
+	_check_item (s_vpn, NM_OPENVPN_KEY_DEV, "tun");
+	_check_item (s_vpn, NM_OPENVPN_KEY_PROTO_TCP, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_COMPRESS, "yes");
+	_check_item (s_vpn, NM_OPENVPN_KEY_FLOAT, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_RENEG_SECONDS, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE, "173.8.149.245:1194");
+	_check_item (s_vpn, NM_OPENVPN_KEY_PORT, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY_DIRECTION, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_CIPHER, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_DATA_CIPHERS, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_AUTH, NULL);
+
+	_check_item (s_vpn, NM_OPENVPN_KEY_CA,   SRCDIR"/ca.crt");
+	_check_item (s_vpn, NM_OPENVPN_KEY_CERT, SRCDIR"/keys/mine.p12");
+	_check_item (s_vpn, NM_OPENVPN_KEY_KEY,  SRCDIR"/keys/mine.p12");
+
+	_check_secret (s_vpn, NM_OPENVPN_KEY_PASSWORD, NULL);
+	_check_secret (s_vpn, NM_OPENVPN_KEY_CERTPASS, NULL);
+}
+
 static void
 test_non_utf8_import (void)
 {
@@ -601,6 +694,7 @@ test_static_key_import (gconstpointer test_data)
 	_check_item (s_vpn, NM_OPENVPN_KEY_TA, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_TA_DIR, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_CIPHER, NULL);
+	_check_item (s_vpn, NM_OPENVPN_KEY_DATA_CIPHERS, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP, "10.8.0.2");
 	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP, "10.8.0.1");
 	_check_item (s_vpn, NM_OPENVPN_KEY_AUTH, NULL);
@@ -712,6 +806,7 @@ test_proxy_http_import (void)
 	_check_item (s_vpn, NM_OPENVPN_KEY_TA, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_TA_DIR, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_CIPHER, "AES-256-CBC");
+	_check_item (s_vpn, NM_OPENVPN_KEY_DATA_CIPHERS, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_AUTH, NULL);
@@ -776,6 +871,7 @@ test_proxy_http_with_auth_import (void)
 	_check_item (s_vpn, NM_OPENVPN_KEY_TA, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_TA_DIR, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_CIPHER, "AES-256-CBC");
+	_check_item (s_vpn, NM_OPENVPN_KEY_DATA_CIPHERS, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_AUTH, NULL);
@@ -814,6 +910,7 @@ test_proxy_socks_import (void)
 	_check_item (s_vpn, NM_OPENVPN_KEY_TA, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_TA_DIR, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_CIPHER, "AES-256-CBC");
+	_check_item (s_vpn, NM_OPENVPN_KEY_DATA_CIPHERS, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_AUTH, NULL);
@@ -905,6 +1002,7 @@ test_route_import (void)
 	gs_unref_object NMConnection *connection = NULL;
 	NMSettingIPConfig *s_ip4;
 	NMSettingVpn *s_vpn;
+	NMIPRoute *route;
 	int num_routes;
 	const char *expected_dest1 = "1.2.3.0";
 	guint32 expected_prefix1   = 24;
@@ -926,81 +1024,168 @@ test_route_import (void)
 
 	s_ip4 = _get_setting_ip4_config (connection);
 
-#if ((NETWORKMANAGER_COMPILATION) & NM_NETWORKMANAGER_COMPILATION_WITH_LIBNM_UTIL)
-	{
-		NMIP4Route *route;
-		const char *expected_nh2   = "0.0.0.0";
+	num_routes = nm_setting_ip_config_get_num_routes (s_ip4);
+	g_assert_cmpint (num_routes, ==, 3);
 
-#define METR(metr) ((metr) == -1 ? 0 : ((guint32) (metr)))
+	route = nm_setting_ip_config_get_route (s_ip4, 0);
+	g_assert_cmpstr (nm_ip_route_get_dest (route), ==, expected_dest1);
+	g_assert_cmpint (nm_ip_route_get_prefix (route), ==, expected_prefix1);
+	g_assert_cmpstr (nm_ip_route_get_next_hop (route), ==, expected_nh1);
+	g_assert_cmpint (nm_ip_route_get_metric (route), ==, expected_metric1);
 
-		num_routes = nm_setting_ip4_config_get_num_routes (s_ip4);
-		g_assert_cmpint (num_routes, ==, 3);
+	route = nm_setting_ip_config_get_route (s_ip4, 1);
+	g_assert_cmpstr (nm_ip_route_get_dest (route), ==, expected_dest2);
+	g_assert_cmpint (nm_ip_route_get_prefix (route), ==, expected_prefix2);
+	g_assert_cmpstr (nm_ip_route_get_next_hop (route), ==, NULL);
+	g_assert_cmpint (nm_ip_route_get_metric (route), ==, expected_metric2);
 
-		route = nm_setting_ip4_config_get_route (s_ip4, 0);
-		g_assert_cmpint (nm_ip4_route_get_dest (route), ==, nmtst_inet4_from_string (expected_dest1));
-		g_assert_cmpint (nm_ip4_route_get_prefix (route), ==, expected_prefix1);
-		g_assert_cmpint (nm_ip4_route_get_next_hop (route), ==, nmtst_inet4_from_string (expected_nh1));
-		g_assert_cmpint (nm_ip4_route_get_metric (route), ==, METR (expected_metric1));
-
-		route = nm_setting_ip4_config_get_route (s_ip4, 1);
-		g_assert_cmpint (nm_ip4_route_get_dest (route), ==, nmtst_inet4_from_string (expected_dest2));
-		g_assert_cmpint (nm_ip4_route_get_prefix (route), ==, expected_prefix2);
-		g_assert_cmpint (nm_ip4_route_get_next_hop (route), ==, nmtst_inet4_from_string (expected_nh2));
-		g_assert_cmpint (nm_ip4_route_get_metric (route), ==, METR (expected_metric2));
-
-		route = nm_setting_ip4_config_get_route (s_ip4, 2);
-		g_assert_cmpint (nm_ip4_route_get_dest (route), ==, nmtst_inet4_from_string (expected_dest3));
-		g_assert_cmpint (nm_ip4_route_get_prefix (route), ==, expected_prefix3);
-		g_assert_cmpint (nm_ip4_route_get_next_hop (route), ==, nmtst_inet4_from_string (expected_nh3));
-		g_assert_cmpint (nm_ip4_route_get_metric (route), ==, METR (expected_metric3));
-	}
-#else
-	{
-		NMIPRoute *route;
-
-		num_routes = nm_setting_ip_config_get_num_routes (s_ip4);
-		g_assert_cmpint (num_routes, ==, 3);
-
-		route = nm_setting_ip_config_get_route (s_ip4, 0);
-		g_assert_cmpstr (nm_ip_route_get_dest (route), ==, expected_dest1);
-		g_assert_cmpint (nm_ip_route_get_prefix (route), ==, expected_prefix1);
-		g_assert_cmpstr (nm_ip_route_get_next_hop (route), ==, expected_nh1);
-		g_assert_cmpint (nm_ip_route_get_metric (route), ==, expected_metric1);
-
-		route = nm_setting_ip_config_get_route (s_ip4, 1);
-		g_assert_cmpstr (nm_ip_route_get_dest (route), ==, expected_dest2);
-		g_assert_cmpint (nm_ip_route_get_prefix (route), ==, expected_prefix2);
-		g_assert_cmpstr (nm_ip_route_get_next_hop (route), ==, NULL);
-		g_assert_cmpint (nm_ip_route_get_metric (route), ==, expected_metric2);
-
-		route = nm_setting_ip_config_get_route (s_ip4, 2);
-		g_assert_cmpstr (nm_ip_route_get_dest (route), ==, expected_dest3);
-		g_assert_cmpint (nm_ip_route_get_prefix (route), ==, expected_prefix3);
-		g_assert_cmpstr (nm_ip_route_get_next_hop (route), ==, expected_nh3);
-		g_assert_cmpint (nm_ip_route_get_metric (route), ==, expected_metric3);
-	}
-#endif
+	route = nm_setting_ip_config_get_route (s_ip4, 2);
+	g_assert_cmpstr (nm_ip_route_get_dest (route), ==, expected_dest3);
+	g_assert_cmpint (nm_ip_route_get_prefix (route), ==, expected_prefix3);
+	g_assert_cmpstr (nm_ip_route_get_next_hop (route), ==, expected_nh3);
+	g_assert_cmpint (nm_ip_route_get_metric (route), ==, expected_metric3);
 }
 
 static void
 test_compress_import (void)
 {
 	_CREATE_PLUGIN (plugin);
-	NMConnection *connection;
+	gs_unref_object NMConnection *connection = NULL;
 	NMSettingVpn *s_vpn;
 
 	connection = get_basic_connection (plugin, SRCDIR, "compress.ovpn");
-	g_assert (connection);
 
-	/* VPN setting */
 	s_vpn = nm_connection_get_setting_vpn (connection);
-	g_assert (s_vpn);
 
-	/* Data items */
+	_check_item (s_vpn, NM_OPENVPN_KEY_ALLOW_COMPRESSION, "asym");
 	_check_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO, "adaptive");
 	_check_item (s_vpn, NM_OPENVPN_KEY_COMPRESS, "lzo");
+}
 
-	g_object_unref (connection);
+static void
+test_push_peer_info_import (void)
+{
+	_CREATE_PLUGIN (plugin);
+	gs_unref_object NMConnection *connection = NULL;
+	NMSettingVpn *s_vpn;
+
+	connection = get_basic_connection (plugin, SRCDIR, "push-peer-info.ovpn");
+
+	s_vpn = nm_connection_get_setting_vpn (connection);
+
+	_check_item (s_vpn, NM_OPENVPN_KEY_PUSH_PEER_INFO, "yes");
+}
+
+static void
+test_proto_udp_import (void)
+{
+	_CREATE_PLUGIN (plugin);
+	gs_unref_object NMConnection *connection = NULL;
+	NMSettingVpn *s_vpn;
+
+	connection = get_basic_connection (plugin, SRCDIR, "proto-udp.ovpn");
+
+	s_vpn = nm_connection_get_setting_vpn (connection);
+
+	_check_item (s_vpn, NM_OPENVPN_KEY_PROTO_TCP, NULL);
+}
+
+static void
+test_proto_udp4_import (void)
+{
+	_CREATE_PLUGIN (plugin);
+	gs_unref_object NMConnection *connection = NULL;
+	NMSettingVpn *s_vpn;
+
+	connection = get_basic_connection (plugin, SRCDIR, "proto-udp4.ovpn");
+
+	s_vpn = nm_connection_get_setting_vpn (connection);
+
+	_check_item (s_vpn, NM_OPENVPN_KEY_PROTO_TCP, NULL);
+}
+
+static void
+test_proto_udp6_import (void)
+{
+	_CREATE_PLUGIN (plugin);
+	gs_unref_object NMConnection *connection = NULL;
+	NMSettingVpn *s_vpn;
+
+	connection = get_basic_connection (plugin, SRCDIR, "proto-udp6.ovpn");
+
+	s_vpn = nm_connection_get_setting_vpn (connection);
+
+	_check_item (s_vpn, NM_OPENVPN_KEY_PROTO_TCP, NULL);
+}
+
+static void
+test_proto_tcp_import (void)
+{
+	_CREATE_PLUGIN (plugin);
+	gs_unref_object NMConnection *connection = NULL;
+	NMSettingVpn *s_vpn;
+
+	connection = get_basic_connection (plugin, SRCDIR, "proto-tcp.ovpn");
+
+	s_vpn = nm_connection_get_setting_vpn (connection);
+
+	_check_item (s_vpn, NM_OPENVPN_KEY_PROTO_TCP, "yes");
+}
+
+static void
+test_proto_tcp4_import (void)
+{
+	_CREATE_PLUGIN (plugin);
+	gs_unref_object NMConnection *connection = NULL;
+	NMSettingVpn *s_vpn;
+
+	connection = get_basic_connection (plugin, SRCDIR, "proto-tcp4.ovpn");
+
+	s_vpn = nm_connection_get_setting_vpn (connection);
+
+	_check_item (s_vpn, NM_OPENVPN_KEY_PROTO_TCP, "yes");
+}
+
+static void
+test_proto_tcp6_import (void)
+{
+	_CREATE_PLUGIN (plugin);
+	gs_unref_object NMConnection *connection = NULL;
+	NMSettingVpn *s_vpn;
+
+	connection = get_basic_connection (plugin, SRCDIR, "proto-tcp6.ovpn");
+
+	s_vpn = nm_connection_get_setting_vpn (connection);
+
+	_check_item (s_vpn, NM_OPENVPN_KEY_PROTO_TCP, "yes");
+}
+
+static void
+test_proto_tcp4_client_import (void)
+{
+	_CREATE_PLUGIN (plugin);
+	gs_unref_object NMConnection *connection = NULL;
+	NMSettingVpn *s_vpn;
+
+	connection = get_basic_connection (plugin, SRCDIR, "proto-tcp4-client.ovpn");
+
+	s_vpn = nm_connection_get_setting_vpn (connection);
+
+	_check_item (s_vpn, NM_OPENVPN_KEY_PROTO_TCP, "yes");
+}
+
+static void
+test_proto_tcp6_client_import (void)
+{
+	_CREATE_PLUGIN (plugin);
+	gs_unref_object NMConnection *connection = NULL;
+	NMSettingVpn *s_vpn;
+
+	connection = get_basic_connection (plugin, SRCDIR, "proto-tcp6-client.ovpn");
+
+	s_vpn = nm_connection_get_setting_vpn (connection);
+
+	_check_item (s_vpn, NM_OPENVPN_KEY_PROTO_TCP, "yes");
 }
 
 /*****************************************************************************/
@@ -1091,6 +1276,118 @@ test_args_parse_line (void)
 
 /*****************************************************************************/
 
+static void test_version(void) {
+	const struct {
+		guint version;
+		char *const data;
+		} test_data[] = {
+	{
+		.version = 20507,
+		.data =
+			"OpenVPN 2.5.7 x86_64-redhat-linux-gnu [SSL (OpenSSL)] "
+			"[LZO] "
+			"[LZ4] "
+			"[EPOLL] [PKCS11] [MH/PKTINFO] [AEAD] built on May 31 "
+			"2022\n"
+			"library versions: OpenSSL 3.0.5 5 Jul 2022, LZO 2.10\n"
+			"Originally developed by James Yonan\n"
+			"Copyright (C) 2002-2022 OpenVPN Inc <sales@openvpn.net>\n"
+			"Compile time defines: enable_async_push=yes "
+			"enable_comp_stub=no "
+			"enable_crypto_ofb_cfb=yes enable_debug=yes "
+			"enable_def_auth=yes "
+			"enable_dependency_tracking=no enable_dlopen=unknown "
+			"enable_dlopen_self=unknown "
+			"enable_dlopen_self_static=unknown "
+			"enable_fast_install=needless enable_fragment=yes "
+			"enable_iproute2=no "
+			"enable_libtool_lock=yes enable_lz4=yes enable_lzo=yes "
+			"enable_management=yes enable_multihome=yes "
+			"enable_pam_dlopen=no "
+			"enable_pedantic=no enable_pf=yes enable_pkcs11=yes "
+			"enable_plugin_auth_pam=yes enable_plugin_down_root=yes "
+			"enable_plugins=yes enable_port_share=yes "
+			"enable_selinux=yes "
+			"enable_shared=yes enable_shared_with_static_runtimes=no "
+			"enable_silent_rules=yes enable_small=no enable_static=yes "
+			"enable_strict=no enable_strict_options=no "
+			"enable_systemd=yes "
+			"enable_werror=no enable_win32_dll=yes "
+			"enable_x509_alt_username=yes "
+			"with_aix_soname=aix with_crypto_library=openssl "
+			"with_gnu_ld=yes "
+			"with_mem_check=no with_openssl_engine=auto "
+			"with_sysroot=no\n"
+			"",
+		},
+		{
+		.version = 20310,
+		.data =
+			"OpenVPN 2.3.10 x86_64-pc-linux-gnu [SSL (OpenSSL)] [LZO] "
+			"[EPOLL] [PKCS11] [MH] [IPv6] built on Jan  9 2019\n"
+			"library versions: OpenSSL 1.0.2g  1 Mar 2016, LZO 2.08\n"
+			"Originally developed by James Yonan\n"
+			"Copyright (C) 2002-2010 OpenVPN Technologies, Inc. "
+			"<sales@openvpn.net>\n"
+			"Compile time defines: enable_crypto=yes "
+			"enable_crypto_ofb_cfb=yes enable_debug=yes "
+			"enable_def_auth=yes enable_dependency_tracking=no "
+			"enable_dlopen=unknown enable_dlopen_self=unknown "
+			"enable_dlopen_self_static=unknown enable_fast_install=yes "
+			"enable_fragment=yes enable_http_proxy=yes "
+			"enable_iproute2=yes enable_libtool_lock=yes "
+			"enable_lzo=yes enable_lzo_stub=no "
+			"enable_maintainer_mode=no enable_management=yes "
+			"enable_multi=yes enable_multihome=yes "
+			"enable_pam_dlopen=no enable_password_save=yes "
+			"enable_pedantic=no enable_pf=yes enable_pkcs11=yes "
+			"enable_plugin_auth_pam=yes enable_plugin_down_root=yes "
+			"enable_plugins=yes enable_port_share=yes "
+			"enable_selinux=no enable_server=yes enable_shared=yes "
+			"enable_shared_with_static_runtimes=no "
+			"enable_silent_rules=no enable_small=no enable_socks=yes "
+			"enable_ssl=yes enable_static=yes enable_strict=no "
+			"enable_strict_options=no enable_systemd=yes "
+			"enable_win32_dll=yes enable_x509_alt_username=yes "
+			"with_crypto_library=openssl with_gnu_ld=yes "
+			"with_mem_check=no with_plugindir='${prefix}/lib/openvpn' "
+			"with_sysroot=no\n"
+			"",
+		},
+	};
+	int i;
+
+	for (i = 0; i < (int)G_N_ELEMENTS(test_data); i++) {
+		g_assert_cmpint(test_data[i].version, ==,
+		                nmovpn_version_parse(test_data[i].data));
+	}
+
+#define _test_version(v_x, v_y, v_z, encoded) \
+	G_STMT_START { \
+		const guint _encoded = (encoded); \
+		const guint _v_x = (v_x); \
+		const guint _v_y = (v_y); \
+		const guint _v_z = (v_z); \
+		guint _v2_x; \
+		guint _v2_y; \
+		guint _v2_z; \
+		\
+		g_assert_cmpint(nmovpn_version_encode(_v_x, _v_y, _v_z), ==, _encoded); \
+		\
+		nmovpn_version_decode(_encoded, &_v2_x, &_v2_y, &_v2_z); \
+		g_assert_cmpint(_v_x, ==, _v2_x); \
+		g_assert_cmpint(_v_y, ==, _v2_y); \
+		g_assert_cmpint(_v_z, ==, _v2_z); \
+	} G_STMT_END
+
+	_test_version(1, 5, 88, 10588);
+	_test_version(2, 5, 0, 20500);
+	_test_version(2, 5, 4, 20504);
+	_test_version(3, 0, 0, 30000);
+}
+
+/*****************************************************************************/
+
 NMTST_DEFINE ();
 
 int main (int argc, char **argv)
@@ -1125,9 +1422,15 @@ int main (int argc, char **argv)
 	_add_test_func_simple (test_tls_import_3);
 	_add_test_func ("tls3-export", test_export_compare, "tls3.ovpn", "tls3.ovpntest");
 
+	_add_test_func_simple (test_tls_import_4);
+	_add_test_func ("tls4-export", test_export_compare, "tls4.ovpn", "tls4.ovpntest");
+
 	_add_test_func_simple (test_pkcs12_import);
 	_add_test_func ("pkcs12-export", test_export_compare, "pkcs12.ovpn", "pkcs12.ovpntest");
 
+	_add_test_func_simple (test_pkcs12_with_ca_import);
+	_add_test_func ("pkcs12-with-ca-export", test_export_compare, "pkcs12-with-ca.ovpn", "pkcs12-with-ca.ovpntest");
+
 	_add_test_func_simple (test_non_utf8_import);
 
 	_add_test_func ("static-import-1", test_static_key_import, "static.ovpn", "static", "1");
@@ -1188,8 +1491,37 @@ int main (int argc, char **argv)
 	_add_test_func_simple (test_compress_import);
 	_add_test_func ("compress-export", test_export_compare, "compress.ovpn", "compress.ovpntest");
 
+	_add_test_func_simple (test_push_peer_info_import);
+	_add_test_func ("push-peer-info-export", test_export_compare, "push-peer-info.ovpn", "push-peer-info.ovpntest");
+
+	_add_test_func_simple (test_proto_udp_import);
+	_add_test_func ("proto-udp-export", test_export_compare, "proto-udp.ovpn", "proto-udp.ovpntest");
+
+	_add_test_func_simple (test_proto_udp4_import);
+	_add_test_func ("proto-udp4-export", test_export_compare, "proto-udp4.ovpn", "proto-udp4.ovpntest");
+
+	_add_test_func_simple (test_proto_udp6_import);
+	_add_test_func ("proto-udp6-export", test_export_compare, "proto-udp6.ovpn", "proto-udp6.ovpntest");
+
+	_add_test_func_simple (test_proto_tcp_import);
+	_add_test_func ("proto-tcp-export", test_export_compare, "proto-tcp.ovpn", "proto-tcp.ovpntest");
+
+	_add_test_func_simple (test_proto_tcp4_import);
+	_add_test_func ("proto-tcp4-export", test_export_compare, "proto-tcp4.ovpn", "proto-tcp4.ovpntest");
+
+	_add_test_func_simple (test_proto_tcp6_import);
+	_add_test_func ("proto-tcp6-export", test_export_compare, "proto-tcp6.ovpn", "proto-tcp6.ovpntest");
+
+	_add_test_func_simple (test_proto_tcp4_client_import);
+	_add_test_func ("proto-tcp4-client-export", test_export_compare, "proto-tcp4-client.ovpn", "proto-tcp4-client.ovpntest");
+
+	_add_test_func_simple (test_proto_tcp6_client_import);
+	_add_test_func ("proto-tcp6-client-export", test_export_compare, "proto-tcp6-client.ovpn", "proto-tcp6-client.ovpntest");
+
 	_add_test_func_simple (test_args_parse_line);
 
+	_add_test_func_simple (test_version);
+
 	result = g_test_run ();
 	if (result != EXIT_SUCCESS)
 		return result;
